@@ -28,9 +28,7 @@
 #'   arrange(desc(n))
 #' }
 src_bigquery <- function(project, dataset, billing = project, max_pages = 10) {
-  if (!requireNamespace("dplyr", quietly = TRUE)) {
-    stop("dplyr is required to use src_bigquery", call. = FALSE)
-  }
+  check_installed("dbplyr")
 
   con <- DBI::dbConnect(
     bigquery(),
@@ -44,27 +42,38 @@ src_bigquery <- function(project, dataset, billing = project, max_pages = 10) {
 }
 
 # registered onLoad
-db_query_fields.BigQueryConnection <- function(con, sql) {
-  if (dbplyr::is.sql(sql)) {
-    ds <- bq_dataset(con@project, con@dataset)
-    fields <- bq_query_fields(sql, con@billing, default_dataset = ds)
-  } else {
-    tb <- as_bq_table(con, sql)
-    fields <- bq_table_fields(tb)
-  }
+tbl.BigQueryConnection <- function(src, from, ...) {
+  src <- dbplyr::src_dbi(src, auto_disconnect = FALSE)
+  tbl <- dplyr::tbl(src, from = from, check_from = FALSE)
 
-  vapply(fields, "[[", "name", FUN.VALUE = character(1))
+  # This is ugly, but I don't see a better way of doing this
+  tb <- as_bq_table(src$con, from)
+  tbl$lazy_query$is_view <- !inherits(from, "sql") &&
+    tryCatch(bq_table_meta(tb, "type")$type == "VIEW", bigrquery_notFound = function(e) TRUE)
+  tbl
 }
 
 # registered onLoad
-db_save_query.BigQueryConnection <- function(con, sql, name, temporary = TRUE, ...) {
+dbplyr_edition.BigQueryConnection <- function(con) 2L
+
+# registered onLoad
+db_compute.BigQueryConnection <- function(con,
+                                          table,
+                                          sql,
+                                          ...,
+                                          overwrite = FALSE,
+                                          temporary = TRUE,
+                                          unique_indexes = list(),
+                                          indexes = list(),
+                                          analyze = TRUE,
+                                          in_transaction = FALSE) {
 
   if (is.null(con@dataset)) {
-    destination_table <- if (!temporary) as_bq_table(con, name)
+    destination_table <- if (!temporary) as_bq_table(con, table)
     tb <- bq_project_query(con@project, sql, destination_table = destination_table)
   } else {
     ds <- bq_dataset(con@project, con@dataset)
-    destination_table <- if (!temporary) as_bq_table(con, name)
+    destination_table <- if (!temporary) as_bq_table(con, table)
 
     tb <- bq_dataset_query(ds,
       query = sql,
@@ -72,25 +81,27 @@ db_save_query.BigQueryConnection <- function(con, sql, name, temporary = TRUE, .
     )
   }
 
-  paste0(tb$project, ".", tb$dataset, ".", tb$table)
+  dbplyr::in_catalog(tb$project, tb$dataset, tb$table)
 }
 
 # registered onLoad
-db_analyze.BigQueryConnection <- function(con, table, ...) {
-  TRUE
-}
-
-# registered onLoad
-db_copy_to.BigQueryConnection <- function(con, table, values,
-                            overwrite = FALSE, types = NULL, temporary = TRUE,
-                            unique_indexes = NULL, indexes = NULL,
-                            analyze = TRUE, ...) {
+db_copy_to.BigQueryConnection <- function(con,
+                                          table,
+                                          values,
+                                          ...,
+                                          overwrite = FALSE,
+                                          types = NULL,
+                                          temporary = TRUE,
+                                          unique_indexes = NULL,
+                                          indexes = NULL,
+                                          analyze = TRUE,
+                                          in_transaction = TRUE) {
 
   if (temporary) {
-    abort("BigQuery does not support temporary tables")
+    cli::cli_abort("BigQuery does not support temporary tables")
   }
 
-  tb <- bq_table(con@project, con@dataset, table)
+  tb <- as_bq_table(con, table)
   write <- if (overwrite) "WRITE_TRUNCATE" else "WRITE_EMPTY"
   bq_table_upload(tb, values, fields = types, write_disposition = write)
 
@@ -105,7 +116,11 @@ collect.tbl_BigQueryConnection <- function(x, ...,
                                            max_connections = 6L,
                                            n = Inf,
                                            warn_incomplete = TRUE) {
-  assert_that(length(n) == 1, n > 0L)
+
+  check_number_whole(n, min = 0, allow_infinite = TRUE)
+  check_number_whole(max_connections, min = 1)
+  check_bool(warn_incomplete)
+
   con <- dbplyr::remote_con(x)
 
   if (op_can_download(x)) {
@@ -149,10 +164,19 @@ op_can_download.lazy_select_query <- function(x) {
   query_is_head_only(x)
 }
 #' @export
-op_can_download.lazy_base_query <- function(x) dbplyr::is.ident(x$x)
+op_can_download.lazy_base_query <- function(x) {
+  if (isTRUE(x$is_view)) {
+    FALSE
+  } else if (inherits(x$x, "sql")) {
+    FALSE
+  } else {
+    dbplyr::is.ident(x$x) || inherits(x$x, "dbplyr_table_ident")
+  }
+}
 
 query_is_head_only <- function(x) {
   if (!inherits(x$x, "lazy_base_remote_query")) return(FALSE)
+  if (!op_can_download(x$x)) return(FALSE)
 
   vars_base <- dbplyr::op_vars(x$x)
   if (!is_select_trivial(x$select, vars_base)) return(FALSE)
@@ -195,6 +219,12 @@ op_table.lazy_select_query <- function(x, con) {
   op_table(x$x)
 }
 
+
+# exported onLoad
+same_src.tbl_BigQueryConnection <- function(x, y) {
+  inherits(y, "tbl_BigQueryConnection")
+}
+
 # SQL translation -------------------------------------------------------------
 
 # Don't import to avoid build-time dependency
@@ -208,7 +238,7 @@ sql_join_suffix.BigQueryConnection <- function(con, ...) {
 }
 
 # registered onLoad
-sql_translate_env.BigQueryConnection <- function(x) {
+sql_translation.BigQueryConnection <- function(x) {
   dbplyr::sql_variant(
     dbplyr::sql_translator(.parent = dbplyr::base_scalar,
 
@@ -227,9 +257,13 @@ sql_translate_env.BigQueryConnection <- function(x) {
       Sys.time = sql_prefix("current_time"),
 
       # Regular expressions
-      grepl = sql_prefix("REGEXP_CONTAINS", 2),
-      gsub = function(match, replace, x) {
-        dbplyr::build_sql("REGEXP_REPLACE", list(x, match, replace))
+      grepl = function(pattern, x) {
+        # https://cloud.google.com/bigquery/docs/reference/standard-sql/string_functions#regexp_contains
+        dbplyr::build_sql("REGEXP_CONTAINS", list(x, pattern))
+      },
+      gsub = function(pattern, replace, x) {
+        # https://cloud.google.com/bigquery/docs/reference/standard-sql/string_functions#regexp_replace
+        dbplyr::build_sql("REGEXP_REPLACE", list(x, pattern, replace))
       },
 
       # Other scalar functions
@@ -247,8 +281,10 @@ sql_translate_env.BigQueryConnection <- function(x) {
       pmax = sql_prefix("GREATEST"),
       pmin = sql_prefix("LEAST"),
 
-      # Median
-      median = function(x) dbplyr::build_sql("APPROX_QUANTILES(", x, ", 2)[SAFE_ORDINAL(2)]")
+      runif = function(n = n(), min = 0, max = 1) {
+        RAND <- NULL # quiet R CMD check
+        dbplyr::sql_runif(RAND(), n = {{ n }}, min = min, max = max)
+      },
     ),
     dbplyr::sql_translator(.parent = dbplyr::base_agg,
       n = function() dplyr::sql("count(*)"),
@@ -259,7 +295,12 @@ sql_translate_env.BigQueryConnection <- function(x) {
       sd =  sql_prefix("STDDEV_SAMP"),
       var = sql_prefix("VAR_SAMP"),
       cor = dbplyr::sql_aggregate_2("CORR"),
-      cov = dbplyr::sql_aggregate_2("COVAR_SAMP")
+      cov = dbplyr::sql_aggregate_2("COVAR_SAMP"),
+
+      # Median
+      median = function(x, na.rm = TRUE) {
+        dbplyr::build_sql("APPROX_QUANTILES(", x, ", 2)[SAFE_ORDINAL(2)]")
+      }
     ),
     dbplyr::sql_translator(.parent = dbplyr::base_win,
       all = dbplyr::win_absent("LOGICAL_AND"),
@@ -270,7 +311,9 @@ sql_translate_env.BigQueryConnection <- function(x) {
       cor = dbplyr::win_absent("CORR"),
       cov = dbplyr::win_absent("COVAR_SAMP"),
 
-      n_distinct = dbplyr::win_absent("n_distinct")
+      n_distinct = dbplyr::win_absent("n_distinct"),
+
+      median = dbplyr::win_absent("median")
     )
   )
 }
